package ui.controller;

import java.awt.Point;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;

import org.json.simple.JSONArray;
import org.json.simple.JSONObject;
import org.json.simple.parser.JSONParser;
import org.json.simple.parser.ParseException;

import classes.CpsEdge;
import classes.CpsNode;
import classes.CpsObject;
import classes.HolonElement;
import classes.HolonObject;
import classes.HolonSwitch;
import classes.HolonTransformer;
import ui.model.Model;
import ui.model.idCounter;

public class LoadController {

	private Model MODEL;
	private CategoryController cgC;
	private CanvasController cvsC;
	private ObjectController objC;

	public LoadController(Model model, CategoryController cg, CanvasController cvs, ObjectController obj) {
		this.MODEL = model;
		this.cgC = cg;
		this.cvsC = cvs;
		this.objC = obj;

	}

	/**
	 * Reads the the JSON File and load the state into the Model
	 * 
	 * @param path
	 * @throws IOException
	 */
	public void readJSON(String path) throws IOException {
		JSONParser parser = new JSONParser();
		MODEL.setCategories(new ArrayList<>());
		MODEL.setObjectsOnCanvas(new ArrayList<>());

		ArrayList<String> obj = new ArrayList<>();
		ArrayList<String> ele = new ArrayList<>();
		ArrayList<String> edge = new ArrayList<>();
		ArrayList<String> gp = new ArrayList<>();

		try {

			JSONObject json = (JSONObject) parser.parse(new FileReader(path));

			for (Object key : json.keySet()) {

				if (key.equals("CG"))
					readCategory((JSONArray) json.get(key));
				else if (key.toString().contains("CGO") || key.toString().contains("CVSO"))
					obj.add(key.toString());
				else if (key.toString().contains("CGE") || key.toString().contains("CVSE"))
					ele.add(key.toString());
				else if (key.toString().contains("EDGE"))
					edge.add(key.toString());
				else if (key.toString().contains("CGGP") || key.toString().contains("CVSGP"))
					gp.add(key.toString());
				else
					idCounter.setCounter(Integer.parseInt(json.get(key.toString()).toString()));
			}

			dispatch(obj, json);
			dispatch(ele, json);
			dispatch(edge, json);
			dispatch(gp, json);

		} catch (ParseException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

	/**
	 * dispatch the Keys into the right processing
	 * 
	 * @param input
	 * @param json
	 */
	public void dispatch(ArrayList<String> input, JSONObject json) {

		for (String str : input) {
			if (str.contains("CGO")) {
				readCategoryObject((JSONArray) json.get(str));
			} else if (str.contains("CVSO")) {
				readCanvasObject((JSONArray) json.get(str));
			} else if (str.contains("CGE") || str.contains("CVSE")) {
				readElement((JSONArray) json.get(str));
			} else if (str.contains("EDGE")) {
				readEdge((JSONArray) json.get(str));
			} else if (str.contains("CGGP") || str.contains("CVSGP")) {
				readElementGraph((JSONArray) json.get(str));
			}
		}

	}

	/**
	 * Read all Categories from the JSON file
	 * 
	 * @param arr
	 */
	public void readCategory(JSONArray arr) {

		Iterator<Object> i = arr.iterator();

		while (i.hasNext()) {
			cgC.addNewCategory(i.next().toString());
		}
	}

	/**
	 * Read all Objects in Category from the JSON File
	 * 
	 * @param arr
	 */
	public void readCategoryObject(JSONArray arr) {
		Iterator<Object> i = arr.iterator();

		String type = i.next().toString();

		if (type.equals("HolonObject")) {
			cgC.addNewHolonObject(cgC.searchCatNode(i.next().toString()), i.next().toString(), new ArrayList<>(),
					i.next().toString());
		} else if (type.equals("HolonTransformer")) {
			cgC.addNewHolonTransformer(cgC.searchCatNode(i.next().toString()), i.next().toString(),
					i.next().toString());
		} else if (type.equals("HolonSwitch")) {
			cgC.addNewHolonSwitch(cgC.searchCatNode(i.next().toString()), i.next().toString(), i.next().toString());
		}

	}

	/**
	 * Read all Objects in Canvas from JSON File
	 * 
	 * @param arr
	 */
	public void readCanvasObject(JSONArray arr) {
		Iterator<Object> i = arr.iterator();
		CpsObject cps = null;

		String type = i.next().toString();

		if (type.equals("HolonObject")) {
			cps = new HolonObject(i.next().toString());
		} else if (type.equals("HolonTransformer")) {
			cps = new HolonTransformer(i.next().toString());
		} else if (type.equals("HolonSwitch")) {
			cps = new HolonSwitch(i.next().toString());
		} else if (type.equals("CpsNode")) {
			cps = new CpsNode(i.next().toString());
		}

		cps.setName(i.next().toString());
		cps.setID(Integer.parseInt(i.next().toString()));
		cps.setImage(i.next().toString());
		cps.setPosition(Integer.parseInt(i.next().toString()), Integer.parseInt(i.next().toString()));

		cvsC.addObjectIntoCanvas(cps);
	}

	/**
	 * Read all Elements in Category and Canvas from JSON File
	 * 
	 * @param arr
	 */
	public void readElement(JSONArray arr) {
		Iterator<Object> i = arr.iterator();

		String sav = i.next().toString();
//		int id = Integer.parseInt(i.next().toString());
//
//		HolonElement ele = new HolonElement(i.next().toString(), Integer.parseInt(i.next().toString()),
//				Float.parseFloat(i.next().toString()));
//		ele.setActive(i.next().);

		if (sav.equals("Canvas")) {
			objC.addNewElementIntoCanvasObject(Integer.parseInt(i.next().toString()), i.next().toString(),
					Integer.parseInt(i.next().toString()), Float.parseFloat(i.next().toString()));
		} else
			objC.addNewElementIntoCategoryObject(sav, i.next().toString(), i.next().toString(),
					Integer.parseInt(i.next().toString()), Float.parseFloat(i.next().toString()));
	}

	/**
	 * 
	 * @param arr
	 */
	public void readEdge(JSONArray arr) {
		Iterator<Object> i = arr.iterator();

		CpsEdge edge = new CpsEdge(objC.searchByID(Integer.parseInt(i.next().toString())),
				objC.searchByID(Integer.parseInt(i.next().toString())));
		edge.setCapacity(Float.parseFloat(i.next().toString()));
		edge.setFlow(Float.parseFloat(i.next().toString()));

		cvsC.addEdgeOnCanvas(edge);
	}

	public void readElementGraph(JSONArray arr) {
		Iterator<Object> i = arr.iterator();

		String sav = i.next().toString();
		HolonElement ele;

		if (sav.equals("Canvas")) {
			ele = objC.searchHolonElement((HolonObject) objC.searchByID(Integer.parseInt(i.next().toString())),
					i.next().toString());
			while (i.hasNext())
				ele.getGraphPoints()
						.add(new Point(Integer.parseInt(i.next().toString()), Integer.parseInt(i.next().toString())));
			System.out.println(ele.getGraphPoints().size());
		} else {
			ele = objC.searchHolonElement(objC.searchHolonObject(i.next().toString(),
					objC.searchCategory(sav, MODEL.getCategories()).getObjects()), i.next().toString());
			while (i.hasNext())
				ele.getGraphPoints()
						.add(new Point(Integer.parseInt(i.next().toString()), Integer.parseInt(i.next().toString())));
		}

	}

}
